@@ -1,9 +1,23 @@
 import enum
-from time import time
 import logging
+from time import time
+from datetime import datetime
+
+import sqlalchemy as sa
 
 from .event import Event
 from .errors import StorageError, AuthenticationError
+from .util import call_from_path
+from .storage import get_metadata
+
+
+AuthTable = sa.Table(
+    'auth',
+    get_metadata(),
+    sa.Column('pubkey', sa.Text(), primary_key=True),
+    sa.Column('roles', sa.Text()),
+    sa.Column('created', sa.DateTime())
+)
 
 
 class Role(enum.Enum):
@@ -26,9 +40,9 @@ class Authenticator:
     def __init__(self, storage, options):
         self.default_roles = set(Role.anonymous.value)
         self.storage = storage
-        self.actions, self.valid_urls, self.enabled = self.parse_options(options)
+        self.actions, self.valid_urls, self.is_enabled = self.parse_options(options)
         self.log = logging.getLogger('nostr_relay.auth')
-        if self.enabled:
+        if self.is_enabled:
             self.log.info("Authentication enabled.")
 
     def parse_options(self, options):
@@ -43,7 +57,7 @@ class Authenticator:
         enabled = options.get('enabled', False)
         return actions, valid_urls, enabled
 
-    def check_auth_event(self, auth_event):
+    def check_auth_event(self, auth_event, challenge):
         """
         Validate the authentication event
         """
@@ -56,8 +70,18 @@ class Authenticator:
             raise AuthenticationError("invalid: Too old")
         elif since <= -600:
             raise AuthenticationError("invalid: Too new")
-        if auth_event.content not in self.valid_urls:
-            raise AuthenticationError("invalid: Wrong domain")
+        found_relay = found_challenge = False
+        for tag in auth_event.tags:
+            if tag[0] == 'relay':
+                if tag[1] not in self.valid_urls:
+                    raise AuthenticationError("invalid: Wrong domain")
+                found_relay = True
+            elif tag[0] == 'challenge':
+                if tag[1] != challenge:
+                    raise AuthenticationError("invalid: Wrong challenge")
+                found_challenge = True
+        if not (found_relay and found_challenge):
+            raise AuthenticationError("invalid: Missing required tags")
 
     async def evaluate_target(self, auth_token, action, target):
         # TODO: implement per-object permissions here
@@ -67,8 +91,9 @@ class Authenticator:
         """
         Get the roles assigned to the public key
         """
-        async with self.storage.db.execute("SELECT roles FROM auth WHERE pubkey = ?", (pubkey, )) as cursor:
-            row = await cursor.fetchone()
+        async with self.storage.db.begin() as conn:
+            result = await conn.execute(sa.select(AuthTable.c.roles).where(AuthTable.c.pubkey == pubkey))
+            row = result.fetchone()
         if row:
             return set(row[0].lower())
         else:
@@ -78,26 +103,26 @@ class Authenticator:
         """
         Return all roles in authentication table
         """
-        async with self.storage.db.execute("SELECT pubkey, roles FROM auth") as cursor:
-            async for pubkey, role in cursor:
+        async with self.storage.db.begin() as conn:
+            result = await conn.stream(sa.select(AuthTable.c.pubkey, AuthTable.c.roles))
+            async for pubkey, role in result:
                 yield pubkey, set((role or '').lower())
 
     async def set_roles(self, pubkey, roles):
         """
         Assign roles to the given public key
         """
-        async with self.storage.db.cursor() as cursor:
-            await cursor.execute("INSERT OR REPLACE INTO auth (pubkey, roles, created) VALUES (?, ?, datetime('now'))", (pubkey, roles))
-        await self.storage.db.commit()
+        async with self.storage.db.begin() as conn:
+            await conn.execute(sa.insert(AuthTable).values(pubkey=pubkey, roles=roles, created=datetime.now()))
 
-    async def authenticate(self, auth_event_json: dict):
+    async def authenticate(self, auth_event_json: dict, challenge: str=''):
         """
         Authenticate, using the authentication event described in NIP-42
 
         This will always return a token which can be used with can_do(auth_token, action)
         """
         auth_event = Event(**auth_event_json)
-        self.check_auth_event(auth_event)
+        self.check_auth_event(auth_event, challenge)
 
         token = {
             'pubkey': auth_event.pubkey,
@@ -115,7 +140,7 @@ class Authenticator:
         target can be any object and will be evaluated by evaluate_target(auth_token, action, target)
         """
         can_do = True
-        if self.enabled:
+        if self.is_enabled:
             if action in self.actions:
                 can_do = bool(self.actions[action].intersection(auth_token.get('roles', self.default_roles)))
                 if can_do and target:
@@ -128,14 +153,6 @@ def get_authenticator(storage, configuration: dict):
     Return an authenticator object, according to the configuration dict
     """
 
-    classpath = configuration.get('authenticator_class', 'nostr_relay.auth:Authenticator')
-    modulename, classname = classpath.split(':', 1)
-    if modulename == 'nostr_relay.auth':
-        classobj = globals()[classname]
-    else:
-        import importlib
-        module = importlib.import_module(modulename)
-        classobj = getattr(module, classname)
-    auth_obj = classobj(storage, configuration)
-    return auth_obj
+    classpath = configuration.get('authenticator_class', 'nostr_relay.auth.Authenticator')
+    return call_from_path(classpath, storage, configuration)
 
