@@ -14,29 +14,42 @@ import asyncio
 import logging
 import time
 import rapidjson
-from .config import Config
+from datetime import datetime, timedelta
+import sqlalchemy as sa
+
+from .errors import VerificationError
+from nostr_relay.storage import get_metadata
 
 
-class VerificationError(Exception):
-    pass
+_metadata = get_metadata()
+
+Verification = sa.Table(
+    'verification',
+    _metadata,
+    sa.Column('id', sa.Integer(), primary_key=True),
+    sa.Column('identifier', sa.Text()),
+    sa.Column('metadata_id', _metadata.tables['events'].c.id.type, sa.ForeignKey('events.id', ondelete='CASCADE')),
+    sa.Column('verified_at', sa.TIMESTAMP()),
+    sa.Column('failed_at', sa.TIMESTAMP())
+)
+sa.Index('identifieridx', Verification.c.identifier)
+sa.Index('metadataidx', Verification.c.metadata_id)
+sa.Index('verifiedidx', Verification.c.verified_at)
+
 
 
 class Verifier:
-    VERIFICATION_QUERY = """
-        SELECT verification.id, identifier, verified_at, failed_at, created_at FROM verification
-        LEFT JOIN event ON verification.metadata_id = event.id 
-        WHERE event.pubkey = ? ORDER BY event.created_at DESC
-    """
-    FAILURE_QUERY = "UPDATE verification SET failed_at = strftime('%s', 'now') WHERE id = ?"
-    SUCCESS_QUERY = "UPDATE verification SET verified_at = strftime('%s', 'now') WHERE id = ?"
 
-
-    def __init__(self):
+    def __init__(self, storage, options:dict = None):
+        self.storage = storage
+        self.options = options or {}
+        self.options.setdefault('update_frequency', 3600)
+        self.options.setdefault('expiration', 86400)
         self.running = True
         self.queue = asyncio.Queue()
         # nip05_verification can be "enabled", "disabled", or "passive"
-        self.is_enabled = Config.nip05_verification == 'enabled'
-        self.should_verify = Config.nip05_verification in ('enabled', 'passive')
+        self.is_enabled = options.get('nip05_verification', '') == 'enabled'
+        self.should_verify = options.get('nip05_verification', '') in ('enabled', 'passive')
         if self.should_verify:
             self.log = logging.getLogger(__name__)
 
@@ -60,25 +73,25 @@ class Verifier:
         return False
 
     def check_allowed_domains(self, domain):
-        if Config.verification_whitelist:
-            return domain in Config.verification_whitelist
-        elif Config.verification_blacklist:
-            return domain not in Config.verification_blacklist
-        elif '/' in domain:
+        if '/' in domain:
             return False
+        if self.options.get('whitelist'):
+            return domain in self.options['whitelist']
+        elif self.options.get('blacklist'):
+            return domain not in self.options['blacklist']
         return True
 
-    async def verify(self, cursor, event):
+    async def verify(self, conn, event):
         """
         Check an event against the NIP-05
         verification table
         """
-        if Config.nip05_verification == 'disabled':
+        if not self.should_verify:
             return True
 
         
         if event.kind == 0:
-            is_candidate = await self.update_metadata(cursor, event)
+            is_candidate = await self.update_metadata(conn, event)
             if not is_candidate:
                 if self.is_enabled:
                     raise VerificationError("rejected: metadata must have nip05 tag")
@@ -87,8 +100,15 @@ class Verifier:
             else:
                 return True
 
-        await cursor.execute(self.VERIFICATION_QUERY, (event.pubkey, ))
-        row = await cursor.fetchone()
+        query = sa.select(Verification.c.id, Verification.c.identifier, Verification.c.verified_at, Verification.c.failed_at, sa.column('events.created_at')).select_from(
+                sa.join(Verification, self.storage.EventTable, Verification.c.metadata_id == self.storage.EventTable.c.id, isouter=True)
+            ).where(
+                (self.storage.EventTable.c.pubkey == bytes.fromhex(event.pubkey))
+            )
+
+        result = await conn.execute(query)
+        row = result.fetchone()
+
         if not row:
             if self.is_enabled:
                 raise VerificationError(f"rejected: pubkey {event.pubkey} must be verified")
@@ -97,8 +117,8 @@ class Verifier:
         else:
             vid, identifier, verified_at, failed_at, created_at = row
             self.log.debug("Checking verification for %s verified:%s created:%s", identifier, verified_at, created_at)
-            now = time.time()
-            if ((verified_at or 0) + Config.verification_expiration) < now:
+            now = datetime.now()
+            if (now - timedelta(seconds=self.options['expiration'])) > verified_at:
                 # verification has expired
                 if self.is_enabled:
                     raise VerificationError(f"rejected: verification expired for {identifier}")
@@ -116,15 +136,8 @@ class Verifier:
         return True
 
     async def verification_task(self, db):
-        self.log.info("Starting verification task. Interval %s", Config.verification_update_frequency)
+        self.log.info("Starting verification task. Interval %s", self.options['update_frequency'])
         last_run = 0
-        query = f"""
-            SELECT v.id, identifier, verified_at, pubkey, metadata_id FROM verification as v
-            LEFT JOIN event ON v.metadata_id = event.id
-            WHERE pubkey IS NOT NULL AND
-            (? - verified_at > {Config.verification_expiration})
-            ORDER BY verified_at DESC
-        """
         while self.running:
             candidate = await self.queue.get()
             if candidate is None:
@@ -132,16 +145,22 @@ class Verifier:
             self.log.debug("Got candidate %s", candidate)
             candidates = []
             try:
-                if (time.time() - last_run) > Config.verification_update_frequency:
+                if (time.time() - last_run) > self.options['update_frequency']:
                     self.log.debug("running batch query")
-                    async with db.cursor() as cursor:
+                    query = sa.select(Verification.c.id, Verification.c.identifier, Verification.c.verified_at, sa.column('events.pubkey'), Verification.c.metadata_id).select_from(
+                        sa.join(Verification, self.storage.EventTable, Verification.c.metadata_id == self.storage.EventTable.c.id, isouter=True)
+                    ).where(
+                        (sa.column('events.pubkey') != None) & (Verification.c.verified_at > (int(time.time() - self.options['expiration'])))
+                    )
+
+                    async with db.begin() as cursor:
                         try:
-                            await cursor.execute(query, (int(time.time()), ))
+                            results = await cursor.stream(query)
+                            async for row in results:
+                                candidates.append(row)
                         except Exception:
                             self.log.exception('batch query')
                             continue
-                        async for row in cursor:
-                            candidates.append(row)
                             # vid, identifier, verified_at, pubkey = row
             except Exception:
                 self.log.exception("batch_query")
@@ -154,33 +173,34 @@ class Verifier:
                 self.log.exception("process_verifications")
             else:
                 if success or failure:
-                    async with db.cursor() as cursor:
+                    async with db.begin() as conn:
                         for vid, identifier, metadata_id in success:
                             if vid is None:
                                 # first time verifying
-                                await cursor.execute("INSERT INTO verification (identifier, metadata_id, verified_at) VALUES (?, ?, strftime('%s', 'now'))", (identifier, metadata_id, ))
+                                await conn.execute(sa.insert(Verification).values({'identifier': identifier, 'metadata_id': metadata_id, 'verified_at': datetime.now()}))
                             else:
-                                await cursor.execute(self.SUCCESS_QUERY, (vid, ))
+                                await conn.execute(sa.update(Verification).where(Verification.c.id == vid).values({'verified_at': datetime.now()}))
                         for vid, identifier, metadata_id in failure:
                             if vid is None:
                                 # don't persist first time candidates
                                 continue
                             else:
-                                await cursor.execute(self.FAILURE_QUERY, (vid, ))
-                        await db.commit()
+                                await conn.execute(sa.update(Verification).where(Verification.c.id == vid).values({'failed_at': datetime.now()}))
                     self.log.info("Saved success:%d failure:%d", len(success), len(failure))
             last_run = time.time()
 
         self.log.info("Stopped verification task")
 
+    def get_aiohttp_session(self):
+        import aiohttp
+        return aiohttp.ClientSession(timeout=aiohttp.ClientTimeout(total=10.0), json_serialize=rapidjson.dumps)
+
     async def process_verifications(self, candidates):
         success = []
         failure = []
-        import aiohttp
-        async with aiohttp.ClientSession(timeout=aiohttp.ClientTimeout(total=10.0), json_serialize=rapidjson.dumps) as session:
-
+        async with self.get_aiohttp_session() as session:
             for vid, identifier, verified_at, pubkey, metadata_id in candidates:
-                LOG.info("Checking verification for %s. Last verified %d", identifier, verified_at)
+                self.log.info("Checking verification for %s. Last verified %d", identifier, verified_at)
                 uname, domain = identifier.split('@', 1)
                 domain = domain.lower()
                 if not self.check_allowed_domains(domain):
@@ -195,6 +215,7 @@ class Verifier:
                     async with session.get(url) as response:
                         data = await response.json(loads=rapidjson.loads)
                     names = data['names']
+                    assert isinstance(names, dict)
                 except Exception:
                     self.log.exception("Failure verifying %s from %s", identifier, url)
                     failure.append([vid, identifier, metadata_id])
@@ -217,4 +238,5 @@ class Verifier:
             self.running = False
             await self.queue.put(None)
 
-
+    def is_processing(self):
+        return not self.queue.empty()
